//
//  Brahe
//  A heterogenous library of mathematical and scientific functions
//
//  gcflcm.h
//
//  Copyright 2006 Scott Robert Ladd
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//      Free Software Foundation, Inc.
//      59 Temple Place - Suite 330
//      Boston, MA 02111-1307, USA.
//
//  For more information on this software package, please visit
//  Scott's web site, Coyote Gulch Productions, at:
//
//      http://www.coyotegulch.com
//
//  Closed-source licenses are available from the author at reasonable
//  prices.
//  

#include "mathtools.h"

// lowest common denominator
uint64_t brahe_lcm(const uint64_t x, const uint64_t y)
{
    uint64_t result = 0;

    if (x == y)
        result = x;
    else
        if (x < y)
            result = (y / brahe_gcf(x,y)) * x;
        else
            result = (x / brahe_gcf(y,x)) * y;

    return result;
}

// greatest common multiple
uint64_t brahe_gcf(uint64_t x, uint64_t y)
{
    uint64_t temp;

    while (y != 0)
    {
        temp = x % y;
        x = y;
        y = temp;
    }

    return x;
}
